/**
 * @file		dbstore_py.cpp
 *
 * @date		2014-10-16
 * @author	s.imazu
 *
 *  例外送出時は"return NULL; " とする。　"Py_RETURN_NONE;" はだめ。
 * 
 */
#if defined(WIN32) || defined(WIN64)
 #pragma warning(disable:4786)
#endif // WIN32

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern "C" {
#include <Python.h>
void	init_dbstore(); 
};

#if defined(WIN32) || defined(WIN64)
 #define	strncasecmp	_strnicmp
 #define	strcasecmp		_stricmp
 typedef	__int64			int64_t;
#else
 #include <string>
 #if PY_VERSION_HEX <= 0x0204FFFF
    typedef	int				Py_ssize_t;
  #endif
#endif // WIN32

#include "dbstore.h"
#define NAME_LEN (32+1)

const int R_PARAM_TYPE_STRING=1;
const int R_PARAM_TYPE_BYTE=2;
const int R_PARAM_TYPE_SHORT=3;
const int R_PARAM_TYPE_INT=4;
const int R_PARAM_TYPE_FLOAT=5;
const int R_PARAM_TYPE_DOUBLE=6;
const int R_PARAM_TYPE_INT64=7;

static PyObject* DBstoreError;
//
//  PyObject* code_obj = Py_BuildValue("(s,s,i)",func_name", "error message ", code);
//  PyErr_SetObject(DBstoreError, code_obj);
//  Py_DECREF(code_obj);  <-- ??????????????
//  return NULL;
//
//   try:
//
//   except labcom._dbstore.Error, (func, msg,code):    
//

//  引数が多い場合は、TypeError例外
//  引数の型が違う場合は、TypeError例外

/**
	[関数名] version

	<引数>
		なし

	<返値>
		ver :string	retrieveのバージョン文字列

 */
static PyObject*
dbstore_version_py(PyObject* self,  PyObject* args)
{
	PyObject* retVal = Py_BuildValue("s", dbsVersion());
	return retVal;
}
static PyObject*
dbstore_zlib_ver_py(PyObject* self,  PyObject* args)
{
	PyObject* retVal = Py_BuildValue("s", dbsZversion());
	return retVal;
}

/**
	[関数名] open

	<引数>
		mail_addr :String
		diag :String
		shot_number :uint
		sub_shot :ushort
		data_type :String ('RAW' or 'ANA')

	<返値>
		dbs_d :int	dbstore記述子（正）

	<例外>
		TypeError,_dbstore.Error
 */
static PyObject*
dbstore_open_py(PyObject* self, PyObject* args)
{
	char* mail_addr;
	char* diag;
	unsigned int shot_number;
	unsigned short sub_shot;
	char* data_type;
  
	if ( !PyArg_ParseTuple(args,"ssIHs", &mail_addr, &diag,  &shot_number, &sub_shot, &data_type) ){
		return NULL;
	}
	short i_type;
	if( 0 == strcasecmp("RAW",data_type) ) {
		i_type = 1;
	}
	else if( 0 == strcasecmp("ANA",data_type)) {
		i_type = 2;
	}
	else {
		PyObject* code_obj = Py_BuildValue("(s,s,s)","Open", "DATA TYPE is unknown." ,  data_type);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}

	int dbs_d = dbsOpen( mail_addr, diag,  shot_number, sub_shot, i_type);
	if( 0 > dbs_d ) {
		PyObject* code_obj = Py_BuildValue("(s,s,i)","Open",  dbsErrorMessage(dbs_d),  dbs_d);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	PyObject* ret_val = Py_BuildValue("i", dbs_d);
	return ret_val;
}
/**
	[関数名] close

	<引数>
		dbs_d :int	dbstore記述子

	<返値>
		code :int		コード（0: success 負:error）

	<例外>
		TypeError,_dbstore.Error
 */
static PyObject*
dbstore_close_py(PyObject* self, PyObject* args)
{
	int dbs_d;
  
	if ( !PyArg_ParseTuple(args,"i", &dbs_d) ){
		return NULL;
	}

	int ret = dbsClose( dbs_d);
	if( 0 > ret) {
		PyObject* code_obj = Py_BuildValue("(s,s,i)","Close",  dbsErrorMessage(ret), ret);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	PyObject* ret_val = Py_BuildValue("i", ret);
	return ret_val;
}
/**
	[関数名] abort

	<引数>
		dbs_d :int	dbstore記述子

	<返値>
		code :int		コード（0: success 負:error）

	<例外>
		TypeError

 dbsAbortでは例外を送出しない

 */
static PyObject*
dbstore_abort_py(PyObject* self, PyObject* args)
{
	int dbs_d;
  
	if ( !PyArg_ParseTuple(args,"i", &dbs_d) ){
		return NULL;
	}

	int ret = dbsAbort( dbs_d);
//	if( 0 > ret) {
//		PyObject* code_obj = Py_BuildValue("(s,s,i)","Abort",  dbsErrorMessage(ret), ret);
//		PyErr_SetObject(DBstoreError, code_obj);
//		Py_DECREF(code_obj);  
//		return NULL;
//	}
	PyObject* ret_val = Py_BuildValue("i",  ret);
	return ret_val;
}

static int
convert_parameter(PyObject* param_obj, int * pp_cnt, char** pp_item[], char** pp_val[], int** pp_type)
{
	char** p_item = NULL;
	char** p_val = NULL;
	int* p_type = NULL;
	int p_cnt = 0;
	bool is_error = false;

	if( Py_None == param_obj ) {
		;	// no-param is not error
	}
	else {
//		if( PyObject_IsInstance(param_obj, (PyObject*)&PyDict_Type) ) {
		// パラメータはDictに格納されている。
		if( !PyDict_Check(param_obj) ) {
			PyObject* code_obj = Py_BuildValue("(s,s)","Write", "PARAMETER  should be a dictionary." );
			PyErr_SetObject(DBstoreError, code_obj);
			Py_DECREF(code_obj);  
			return 0;
		}
	
		int dict_size = PyDict_Size(param_obj);
		if( 0 < dict_size ) {
			p_item = new char*[dict_size];
			p_val = new char*[dict_size];
			p_type = new int[dict_size];
			for(int i=0;i<dict_size;i++) {
				p_item[i] = p_val[i] = NULL; 
			}
		}

		PyObject* key = NULL;
		PyObject* value = NULL;
//		int pos = 0;
		Py_ssize_t pos=0;
		int ktmp_len;
		int vtmp_len;
		char* ktmp = NULL;
		char* vtmp = NULL;
		char* ptmp = NULL;
		long	ltmp;
		long long	lltmp;
		double dtmp;
		while( PyDict_Next(param_obj, &pos, &key, &value) ) {
			// keyは文字列である。
			if( !PyString_Check(key) ) {
					is_error = true;
					break;
			}
			ktmp_len = PyString_Size(key);
			ktmp = new char[ktmp_len+1];
			memcpy(ktmp, PyString_AsString(key), ktmp_len);
			ktmp[ktmp_len]='\0';
			// #PTYPEはデータタイプが格納されている
			ptmp = strstr(ktmp, "#PTYPE");
			if( NULL !=  ptmp ) {
				*ptmp='\0';
				for(int loop=p_cnt-1; -1 < loop;loop--) {
					if( 0 == strcmp(p_item[loop], ktmp) ) {
 						p_type[loop] = (int)PyInt_AsLong(value);
						break;
					}
				}
				delete [] ktmp;
				ktmp = NULL;
				continue;
			}

			p_item[p_cnt] = ktmp;
			if( PyString_Check(value) ) {
				vtmp_len = PyString_Size(value);
				vtmp = new char[vtmp_len+1];
				memcpy(vtmp, PyString_AsString(value), vtmp_len);
				vtmp[vtmp_len]='\0';
				p_val[p_cnt] = vtmp;
				p_type[p_cnt] = R_PARAM_TYPE_STRING;
			}
			else if( PyInt_Check(value) ) {
				ltmp = PyInt_AsLong(value);
				if( -1 == ltmp ) {
					if( PyErr_Occurred() ) {
						is_error = true;
						break;
					}
				}
				vtmp = new char[NAME_LEN];
				sprintf(vtmp, "%ld", ltmp);
				p_val[p_cnt] = vtmp;
				if( 4 == sizeof(ltmp) ) {
					p_type[p_cnt] = R_PARAM_TYPE_INT;
				}
				else {
					int64_t llmax = 0x7fffffff;
					int64_t llmin = (llmax+1)*-1;
					if( llmax < ltmp || llmin > ltmp) 
						p_type[p_cnt] = R_PARAM_TYPE_INT64;
					else 
						p_type[p_cnt] = R_PARAM_TYPE_INT;
				}
			}
			else if( PyLong_Check(value) ) {

/******* この手法でINT64がきた場合、PyLong_AsLong( value);で例外が発生するが
　　　　Python2.4では、うまく補足できなかった。（やりかたが悪いかも）
　　　　また、例外が浮いてしまい、GC時にわるさをするようにも、見受けられた。
　　　　その場ではわからず、時間を費したので、ここに明記する。 2014-10-16 by s.imazu
　　　　　
				int ovf = 0;
				try {
#if PY_MINOR_VERSION < 7
					ltmp = PyLong_AsLong( value);
#else
					ltmp = PyLong_AsLongAndOverflow( value, &ovf);
#endif
				}
				catch ( ... ) {
					ovf = 1;
					// overflow
				}
				if( 0 == ovf && ltmp != -1) {
					vtmp = new char[32];
					sprintf(vtmp, "%ld", ltmp);
					p_val[p_cnt] = vtmp;
					if( 4 == sizeof(ltmp) ) {
						p_type[p_cnt] = R_PARAM_TYPE_INT;
					}
					else {
						int64_t llmax = 0x7fffffff;
						int64_t llmin = (llmax+1)*-1;
						if( llmax < ltmp || llmin > ltmp) 
							p_type[p_cnt] = R_PARAM_TYPE_INT64;
						else 
							p_type[p_cnt] = R_PARAM_TYPE_INT;
					}
				}
				else {
					lltmp = PyLong_AsLongLong(value);
					vtmp = new char[32];
					sprintf(vtmp, "%lld", lltmp);
					p_val[p_cnt] = vtmp;
					int64_t llmax = 0x7fffffff;
					int64_t llmin = (llmax+1)*-1;
					if( llmax < lltmp || llmin > lltmp) 
						p_type[p_cnt] = R_PARAM_TYPE_INT64;
					else 
						p_type[p_cnt] = R_PARAM_TYPE_INT;
				}
******/
				lltmp = PyLong_AsLongLong(value);
				vtmp = new char[NAME_LEN];
				sprintf(vtmp, "%lld", lltmp);
				p_val[p_cnt] = vtmp;
				int64_t llmax = 0x7fffffff;
				int64_t llmin = (llmax+1)*-1;
				if( llmax < lltmp || llmin > lltmp) 
					p_type[p_cnt] = R_PARAM_TYPE_INT64;
				else 
					p_type[p_cnt] = R_PARAM_TYPE_INT;
			}
			else if( PyFloat_Check(value) ) {
				dtmp = PyFloat_AsDouble(value);
				vtmp = new char[NAME_LEN];
				sprintf(vtmp, "%f", dtmp);
				p_val[p_cnt] = vtmp;
				p_type[p_cnt] = R_PARAM_TYPE_DOUBLE;
			}
			else {
				is_error  = true;
				break;
			}
			p_cnt++;
			vtmp = NULL; 
			ktmp = NULL;
		}	// End-While

		if( is_error ) {
			if( NULL != vtmp ) delete [] vtmp;
			if( NULL != ktmp ) delete [] ktmp;
			for( int i=0;i<p_cnt;i++ ) {
				if( NULL != p_item[i] ) delete [] p_item[i];
				if( NULL != p_val[i] )   delete [] p_val[i];
			}
			if( NULL != p_item ) delete [] p_item;
			if( NULL != p_val ) delete [] p_val;
			if( NULL != p_type ) delete [] p_type;
			PyObject* code_obj = Py_BuildValue("(s,s)","Write", "PARAMETER  cannot convert." );
			PyErr_SetObject(DBstoreError, code_obj);
			Py_DECREF(code_obj);  
			return 0;
		}
	
	} // param
	*pp_cnt = p_cnt;
	*pp_item = p_item;
	*pp_val = p_val;
	*pp_type = p_type;

	return 1;
}
/**
	[関数名] write

	<引数>
		dbs_d :int					dbstore記述子
		channel_number :int	チャネル番号
		parameters :Dict			パラメータ
		data :PyString				データ
		image_type :String		イメージタイプ

	<返値>
		code :int		コード（0: success 負:error）

	<例外>
		TypeError

   TODO None

 */
static PyObject*
dbstore_write_py(PyObject* self, PyObject* args)
{
	int dbs_d = 0;
  	int ch_no = 0;
	PyObject* param_obj;
	PyObject* data_obj;
	char* image_type;

	if ( !PyArg_ParseTuple(args,"iiOOs", &dbs_d, &ch_no, &param_obj, &data_obj, &image_type) ){
		return NULL;
	}
	char* block_data = NULL;
	unsigned int block_data_len = 0;
	if( Py_None == data_obj ) {
		// no-data is not error
	}
	else {
//		if( PyObject_IsInstance(data_obj, (PyObject*)&PyString_Type) ) {
		if( PyString_Check(data_obj) ) {
			block_data = PyString_AsString(data_obj);
			block_data_len = PyString_Size(data_obj);
		}
		else {
			PyObject* code_obj = Py_BuildValue("(s,s)","Write", "DATA  should be a binary string." );
			PyErr_SetObject(DBstoreError, code_obj);
			Py_DECREF(code_obj);  
			return NULL;
		}
	}

	char** p_item = NULL;
	char** p_val = NULL;
	int* p_type = NULL;
	int p_cnt = 0;

	int ret = convert_parameter( param_obj, &p_cnt, &p_item, &p_val, &p_type);
	if( 0 == ret ) {
			return NULL;
	}
	if( 0 == p_cnt && NULL == block_data ) {
		// both no data are error.
		PyObject* code_obj = Py_BuildValue("(s,s)","Write",  "No data and no parameter.");
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	ret = dbsWrite2( dbs_d,ch_no, p_cnt, p_item, p_val, p_type, block_data_len, (unsigned char*)block_data, image_type);

	for( int i=0;i<p_cnt;i++ ) {
		if( NULL != p_item[i] ) delete [] p_item[i];
		if( NULL != p_val[i] ) delete [] p_val[i];
	}
	if( NULL != p_item ) delete [] p_item;
	if( NULL != p_val ) delete [] p_val;
	if( NULL != p_type ) delete [] p_type;

	if( 0 > ret) {
		PyObject* code_obj = Py_BuildValue("(s,s,i)","Write",  dbsErrorMessage(ret), ret);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	PyObject* ret_val = Py_BuildValue("i",  ret);
	return ret_val;
}
/**
	[関数名] open_frame

	<引数>
		dbs_d :int					dbstore記述子
		channel_number :int	チャネル番号
		x_size :uint		
		y_size :uint		
		data_type :String		パラメータ

	<返値>
		code :int		コード（0: success 負:error）

	<例外>
		TypeError
 */
static PyObject*
dbstore_open_frame_py(PyObject* self, PyObject* args)
{
	int dbs_d;
	int ch_no;
	unsigned int x_size;
	unsigned int y_size;
	char* d_type;
  
	if ( !PyArg_ParseTuple(args,"iiIIs", &dbs_d, &ch_no, &x_size, &y_size, &d_type) ){
		return NULL;
	}

	int ret = dbsOpenFrame( dbs_d, ch_no,  x_size,  y_size, d_type);
	if( 0 > ret ) {
		PyObject* code_obj = Py_BuildValue("(s,s,i)","OpenFrame",  dbsErrorMessage(ret),  ret);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	PyObject* ret_val = Py_BuildValue("i", ret);
	return ret_val;
}
/**
	[関数名] close_frame

	<引数>
		dbs_d :int					dbstore記述子
		channel_number :int	チャネル番号
		parameters :Dict		

	<返値>
		code :int		コード（0: success 負:error）

	<例外>
		TypeError
 */
static PyObject*
dbstore_close_frame_py(PyObject* self, PyObject* args)
{
	int dbs_d;
	int ch_no;
	PyObject* param_obj;
  
	if ( !PyArg_ParseTuple(args,"iiO", &dbs_d, &ch_no, &param_obj) ){
		return NULL;
	}

	char** p_item = NULL;
	char** p_val = NULL;
	int* p_type = NULL;
	int p_cnt = 0;

	int ret = convert_parameter( param_obj, &p_cnt, &p_item, &p_val, &p_type);
	if( 0 == ret ) {
			return NULL;
	}

	ret = dbsCloseFrame4( dbs_d, ch_no, p_cnt, p_item, p_val, p_type);

	for( int i=0;i<p_cnt;i++ ) {
		if( NULL != p_item[i] ) delete [] p_item[i];
		if( NULL != p_val[i] ) delete [] p_val[i];
	}
	if( NULL != p_item ) delete [] p_item;
	if( NULL != p_val ) delete [] p_val;
	if( NULL != p_type ) delete [] p_type;

	if( 0 > ret ) {
		PyObject* code_obj = Py_BuildValue("(s,s,i)","CloseFrame",  dbsErrorMessage(ret),  ret);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	PyObject* ret_val = Py_BuildValue("i", ret);
	return ret_val;
}
/**
	[関数名] write_frame

	<引数>
		dbs_d :int					dbstore記述子
		channel_number :int	チャネル番号
		frame_number :int	チャネル番号
		data :PyString		

	<返値>
		code :int		コード（0: success 負:error）

	<例外>
		TypeError
 */
static PyObject*
dbstore_write_frame_py(PyObject* self, PyObject* args)
{
	int dbs_d;
	int ch_no;
	unsigned int fr_no;
	PyObject* data_obj;
  
	if ( !PyArg_ParseTuple(args,"iiiO", &dbs_d, &ch_no, &fr_no, &data_obj) ){
		return NULL;
	}

	char* block_data = NULL;
	unsigned int block_data_len = 0;
	if( Py_None == data_obj ) {
		// no-data is not error
	}
	else {
		if( PyString_Check(data_obj) ) {
			block_data = PyString_AsString(data_obj);
			block_data_len = PyString_Size(data_obj);
		}
		else {
			PyObject* code_obj = Py_BuildValue("(s,s)","WriteFrame", "DATA  should be a binary string." );
			PyErr_SetObject(DBstoreError, code_obj);
			Py_DECREF(code_obj);  
			return NULL;
		}
	}

	int ret = dbsWriteFrame( dbs_d, ch_no, fr_no, block_data_len, block_data);
	if( 0 > ret) {
		PyObject* code_obj = Py_BuildValue("(s,s,i)","WriteFrame",  dbsErrorMessage(ret), ret);
		PyErr_SetObject(DBstoreError, code_obj);
		Py_DECREF(code_obj);  
		return NULL;
	}
	PyObject* ret_val = Py_BuildValue("i",  ret);
	return ret_val;
}

static PyMethodDef dbstore_methods[] = {
  {"open",			dbstore_open_py,			METH_VARARGS},
  {"close",			dbstore_close_py,			METH_VARARGS},
  {"abort",			dbstore_abort_py,			METH_VARARGS},
  {"write",			dbstore_write_py,			METH_VARARGS},
  {"open_frame",dbstore_open_frame_py,	METH_VARARGS},
  {"close_frame",dbstore_close_frame_py,	METH_VARARGS},
  {"write_frame",dbstore_write_frame_py,	METH_VARARGS},

  {"version",			dbstore_version_py,			METH_NOARGS},
  {"zlib_version",	dbstore_zlib_ver_py,			METH_NOARGS},
  {NULL},
};

void init_dbstore() 
{
  //  import_array();
  PyObject *m,*d;
  
  m = Py_InitModule("labcom._dbstore", dbstore_methods);
  d = PyModule_GetDict(m);
  DBstoreError = Py_BuildValue("s", "labcom._dbstore.Error");
  PyDict_SetItemString(d,"error", DBstoreError);
}
