#include <octave/oct.h>
#include <octave/ov-struct.h>
#include "retrieve-oct.h"

/*
RETRIEVE_API int retrieveGetDTSParameters2(char *IndexServer,
		char *hostname,	char *modname, unsigned int shot, unsigned short subshot,
		unsigned short channel,	unsigned short paramterCount,
		void *param_name, void *param_value, int *param_type);
*/
DEFUN_DLD (cRetrieveGetDTSParameters2, args, nargout,
"   [code param_struct(Pname,Pvalue,Ptype)] ...\n"
"      = cRetrieveGetDTSParameters2( index_server, host_name, mod_name, ...\n"
"              shot, subshot, channel, param_count, verbose)\n"
           )
{
  const int MIN_IN_ARG = 7;
  const int MAX_IN_ARG = 8;
  const int MAX_OUT_ARG = 2;
  const std::string funcn = "cRetrieveGetDTSParameters2";

  std::string index_server = "";
  std::string host_name = "";
  std::string mod_name = "";
  unsigned int shot_number = 0;
  unsigned short sub_shot = 0;
  unsigned short channel = 0;
  unsigned short param_count = 0;
  bool opt_verbose = false;

  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
      
    // check 1.index_server
    if( 0 != (code=get_arg_string( args(0), index_server )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" index_server:" << index_server << "\n";
    // check 2.host_name
    if( 0 != (code=get_arg_string( args(1), host_name )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" host_name:" << host_name << "\n";
    // check 3.module_name
    if( 0 != (code=get_arg_string( args(2), mod_name )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" module_name:" << mod_name << "\n";
    // check 4.shot_number
    if( 0 != (code=get_arg_uint( args(3), shot_number )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 5.sub_shot
    if( 0 != (code=get_arg_ushort( args(4), sub_shot )) )  {
      code += OCT_IF_ERR_ARG5;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 6.channel
    if( 0 != (code=get_arg_ushort( args(5), channel )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" channel:" << channel << "\n";
    // check 7.param_count
    if( 0 != (code=get_arg_ushort( args(6), param_count )) )  {
      code += OCT_IF_ERR_ARG7;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" param_count:" << param_count << "\n";

    if( nargin > 7 ) {
      // check 8.verbose
      if( 0 != (code=get_arg_bool( args(7), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG8;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )

  int ptype[param_count];
  char pname[param_count][64];    // 64 fixed
  char pval[param_count][128];    // 128 fixed
  
  if( 0 == code ) {
    code = retrieveGetDTSParameters2( index_server.c_str(), host_name.c_str(), mod_name.c_str(),
    shot_number, sub_shot, channel, param_count,&pname, &pval, ptype);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) {
    if( 0 == code ) {
      Cell pname_cell(param_count,1);
      Cell pval_cell(param_count,1);
      Cell ptype_cell(param_count,1);
      for( int x=0;x<param_count;x++ ) {
        pname_cell(x) = &(pname[x][0]);
        pval_cell(x) = &(pval[x][0]);
        ptype_cell(x) = ptype[x];
      }
      octave_map m(dim_vector(param_count,1));
      m.assign("Pname", pname_cell);
      m.assign("Pvalue", pval_cell);
      m.assign("Ptype", ptype_cell);
      ret_vals(1) = m;
    }
    else {
      ret_vals(1) = 0;
    }   
  }
  
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
  }
  return(ret_vals);
}
