#include <octave/oct.h>
#include <octave/int32NDArray.h>
#include "retrieve-oct.h"

/*
 * RETRIEVE_API int retrieveGetDTSParametersCount(char *IndexServer,char *hostname,char *modname,
 *			unsigned int shot,	unsigned short subshot,
 *			short* param_count,	int* start_ch, int* end_ch, char *true_module_name);
 */
DEFUN_DLD (cRetrieveGetDTSParametersCount, args, nargout,
"   [code param_count r_start_ch r_end_ch true_module_name] ...\n"
"      = cRetrieveGetDTSParametersCount( index_server, host_name, mod_name, ...\n"
"                                      shot, subshot, start_ch, end_ch, verbose)\n"
           )
{
  const int MIN_IN_ARG = 7;
  const int MAX_IN_ARG = 8;
  const int MAX_OUT_ARG = 5;
  const std::string funcn = "cRetrieveGetDTSParametersCount";

  std::string index_server = "";
  std::string host_name = "";
  std::string mod_name = "";
  unsigned int shot_number = 0;
  unsigned short sub_shot = 0;
  short param_count = 0;
  int start_ch = 0;
  int end_ch = 0;
  bool opt_verbose = false;
  char x_mod_name[128] = {'\0'};

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
      
    // check 1.index_server
    if( 0 != (code=get_arg_string( args(0), index_server )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" index_server:" << index_server << "\n";
    // check 2.host_name
    if( 0 != (code=get_arg_string( args(1), host_name )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" host_name:" << host_name << "\n";
    // check 3.module_name
    if( 0 != (code=get_arg_string( args(2), mod_name )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" module_name:" << mod_name << "\n";
    // check 4.shot_number
    if( 0 != (code=get_arg_uint( args(3), shot_number )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 5.sub_shot
    if( 0 != (code=get_arg_ushort( args(4), sub_shot )) )  {
      code += OCT_IF_ERR_ARG5;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 6.start_channel
    if( 0 != (code=get_arg_int( args(5), start_ch )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" start_channel:" << start_ch << "\n";
    // check 7.end_channel
    if( 0 != (code=get_arg_int( args(5), end_ch )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" end_channel:" << end_ch << "\n";

    if( nargin > 7 ) {
      // check 8.verbose
      if( 0 != (code=get_arg_bool( args(7), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG8;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )

  if( 0 == code ) {
    code = retrieveGetDTSParametersCount( index_server.c_str(), host_name.c_str(), mod_name.c_str(),
    shot_number, sub_shot, &param_count, &start_ch, &end_ch, x_mod_name);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = param_count;
  if( 2 < nargout ) ret_vals(2) = start_ch;
  if( 3 < nargout ) ret_vals(3) = end_ch;
  if( 4 < nargout ) ret_vals(4) = x_mod_name;

  if( 0 > code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
  }

  return(ret_vals);
}
