#include <octave/oct.h>
#include <octave/dNDArray.h>
#include <octave/fNDArray.h>
#include "retrieve-oct.h"

/*
int retrieveGetDTSdatax2_ex(char* IndexServer,
			char* DTSsource, char* DTShostID, char* DTSmoduleID, char* DTStriggerChannel,
			char* clkSource, char* clkHostID, char* clkModuleID, char* DTSclockChannel,
			int* DTSuserDefine,	int64_t  DTStimeArraySize,
			unsigned int shot,	unsigned short subshot,
			char* clockSource,	char* InternalClock,
			char* strSamplingInterval, char* strPreSampling, int64_t	lastCount,
			short bSilent, int is_double,

			void* timeArray, void* clockCycle, void* triggerTiming )
*/
DEFUN_DLD (cRetrieveGetDTSdatax2_ex, args, nargout,
"   [code clock_cycle trigger_timing rtn_user_define time_array  ] ...\n"
"     = cRetrieveGetDTSdatax2_ex( index_server, ...\n"
"         dts_source, dts_hotsid, dts_moduleid, dts_trig_ch, ...\n"
"         clk_source, clk_hotsid, clk_moduleid, dts_clock_ch, ...\n"
"         dts_user_define, time_array_size, shot_number, sub_shot, ...\n"
"         int_or_ext, internal_clock, s_sampling_interval, s_pre_samples, ...\n"
"         last_count, b_silent, is_double,  verbose)\n"
           )
{
  const int MIN_IN_ARG = 20;
  const int MAX_IN_ARG = 21;
  const int MAX_OUT_ARG = 5;
  const std::string funcn = "cRetrieveGetDTSdatax2_ex";

  std::string index_server = "";
  std::string dts_source = "";
  std::string dts_hostid = "";
  std::string dts_moduleid = "";
  std::string dts_trig_ch = "";
  std::string clk_source = "";
  std::string clk_hostid = "";
  std::string clk_moduleid = "";
  std::string dts_clock_ch = "";
  int dts_user_define = 0;
  int64_t time_array_size = 0;
  unsigned int shot_number = 0;
  unsigned short sub_shot = 0;
  std::string int_or_ext = "";
  std::string internal_clock = "";
  std::string s_sampling_interval = "";
  std::string s_pre_samples = "";
  int64_t last_count = 0;
  short b_silent = 0;
  int is_double = 1;
  bool opt_verbose = false;
  int64_t tmp64v=0;

  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
      
    // check 1.index_server
    if( 0 != (code=get_arg_string( args(0), index_server )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" index_server:" << index_server << "\n";
    // check 2.dts_source
    if( 0 != (code=get_arg_string( args(1), dts_source )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dts_source:" << dts_source << "\n";
    // check 3.dts_hotsid
    if( 0 != (code=get_arg_string( args(2), dts_hostid )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dts_hostid:" << dts_hostid << "\n";
    // check 4.dts_moduleid
    if( 0 != (code=get_arg_string( args(3), dts_moduleid )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dts_moduleid:" << dts_moduleid << "\n";
    // check 5.dts_trig_ch
    if( 0 != (code=get_arg_string( args(4), dts_trig_ch )) )  {
      code += OCT_IF_ERR_ARG5;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dts_trig_ch:" << dts_trig_ch << "\n";
    // check 6.clk_source
    if( 0 != (code=get_arg_string( args(5), clk_source )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" clk_source:" << clk_source << "\n";
    // check 7.clk_hostid
    if( 0 != (code=get_arg_string( args(6), clk_hostid )) )  {
      code += OCT_IF_ERR_ARG7;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" clk_hostid:" << clk_hostid << "\n";
    // check 8.clk_moduleid
    if( 0 != (code=get_arg_string( args(7), clk_moduleid )) )  {
      code += OCT_IF_ERR_ARG8;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" clk_moduleid:" << clk_moduleid << "\n";
    // check 9.dts_clock_ch
    if( 0 != (code=get_arg_string( args(8), dts_clock_ch )) )  {
      code += OCT_IF_ERR_ARG9;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dts_clock_ch:" << dts_clock_ch << "\n";
    // check 10.dts_user_define
    if( 0 != (code=get_arg_int( args(9), dts_user_define )) )  {
      code += OCT_IF_ERR_ARG10;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dts_user_define:" << dts_user_define << "\n";
    // check 11.time_array_size
    if( 0 != (code=get_arg_int64( args(10), time_array_size )) )  {
      code += OCT_IF_ERR_ARG11;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" time_array_size:" << time_array_size << "\n";
    // check 12.shot_number
    if( 0 != (code=get_arg_uint( args(11), shot_number )) )  {
      code += OCT_IF_ERR_ARG12;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" shot_number:" << shot_number << "\n";
    // check 13.sub_shot
    if( 0 != (code=get_arg_ushort( args(12), sub_shot )) )  {
      code += OCT_IF_ERR_ARG13;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" sub_shot:" << sub_shot << "\n";
    // check 14.int_or_ext
    if( 0 != (code=get_arg_string( args(13), int_or_ext )) )  {
      code += OCT_IF_ERR_ARG14;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" int_or_ext:" << int_or_ext << "\n";
    // check 15.internal_clock
    if( 0 != (code=get_arg_string( args(14), internal_clock )) )  {
      code += OCT_IF_ERR_ARG15;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" internal_clock:" << internal_clock << "\n";
    // check 16.s_sampling_interval
    if( 0 != (code=get_arg_string( args(15), s_sampling_interval )) )  {
      code += OCT_IF_ERR_ARG16;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" s_sampling_interval:" << s_sampling_interval << "\n";
    // check 17.s_pre_samples
    if( 0 != (code=get_arg_string( args(16), s_pre_samples )) )  {
      code += OCT_IF_ERR_ARG17;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" s_pre_samples:" << s_pre_samples << "\n";
    // check 18.last_count
    if( 0 != (code=get_arg_int64( args(17), last_count )) )  {
      code += OCT_IF_ERR_ARG18;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" last_count:" << last_count << "\n";
    // check 19.b_silent
    if( 0 != (code=get_arg_short( args(18), b_silent )) )  {
      code += OCT_IF_ERR_ARG19;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" b_silent:" << b_silent << "\n";
    // check 20.is_double
    if( 0 != (code=get_arg_int( args(19), is_double )) )  {
      code += OCT_IF_ERR_ARG20;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" is_double:" << is_double << "\n";
    if( nargin > 20 ) {
      // check 21.verbose
      if( 0 != (code=get_arg_bool( args(20), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG21;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  void* v_time_array = NULL;
  double clock_cycle = 0;
  double trigger_timing = 0;
  float fclock_cycle = 0;
  float ftrigger_timing = 0;
  void* p_clock_cycle;
  void* p_trigger_timing;
  if( 1 == is_double ) {
    p_clock_cycle   = (void*)&clock_cycle;
    p_trigger_timing= (void*)&trigger_timing;
  }
  else {
    p_clock_cycle   = (void*)&fclock_cycle;
    p_trigger_timing= (void*)&ftrigger_timing;
  }
  for( ;0 == code; ) {
    if( 4 < nargout ) {
      if( 1 == is_double )  { v_time_array = new double[time_array_size];}
      else                  { v_time_array = new float[time_array_size];}
      if( NULL == v_time_array ) {
        code = OCT_RTR_ERR_MEMORY_ALLOCATE;
        break;
      }
    }
    
    code = retrieveGetDTSdatax2_ex( (char*)index_server.c_str(),
           (char*)dts_source.c_str(), (char*)dts_hostid.c_str(), 
           (char*)dts_moduleid.c_str(), (char*)dts_trig_ch.c_str(),
           (char*)clk_source.c_str(), (char*)clk_hostid.c_str(),
           (char*)clk_moduleid.c_str(), (char*)dts_clock_ch.c_str(),
           &dts_user_define, time_array_size, shot_number, sub_shot,
            (char*)int_or_ext.c_str(), (char*)internal_clock.c_str(),
            (char*)s_sampling_interval.c_str(), (char*)s_pre_samples.c_str(),
            last_count, b_silent, is_double,v_time_array, p_clock_cycle, p_trigger_timing);
    break; 
  } // End of for( ;; )
  
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 3 < nargout ) ret_vals(3) = dts_user_define;
  if( 1 == is_double ) {
    if( 1 < nargout ) ret_vals(1) = clock_cycle;
    if( 2 < nargout ) ret_vals(2) = trigger_timing;
    double* time_array = (double*)v_time_array;
    if( 4 < nargout ) {
      if( NULL == time_array || 0 != code ) {
        ret_vals(4) = 0;
      }
      else {
        NDArray dd(dim_vector(time_array_size,1));
        for( int64_t x=0; x< time_array_size; x++ ) {
           dd(x) = *(time_array+x) ;
        }
        ret_vals(4) = dd;
      }
    }
    if( NULL != time_array) delete [] time_array;
  }
  else {
    if( 1 < nargout ) ret_vals(1) = fclock_cycle;
    if( 2 < nargout ) ret_vals(2) = ftrigger_timing;
    float* time_array = (float*)v_time_array;
    if( 4 < nargout ) {
      if( NULL == time_array || 0 != code ) {
        ret_vals(4) = 0;
      }
      else {
        FloatNDArray dd(dim_vector(time_array_size,1));
        for( int64_t x=0; x< time_array_size; x++ ) {
           dd(x) = *(time_array+x) ;
        }
        ret_vals(4) = dd;
      }
    }
    if( NULL != time_array) delete [] time_array;
  }
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
  }

  return(ret_vals);
}
