#include <octave/oct.h>
#include "retrieve-oct.h"

DEFUN_DLD (changeSite, args, nargout,
" Change Site for retrieve. \n"
"   chnagesite site_name \n"
"   code = changeSite( site_name ) \n"
"   code = changeSite( site_name, 'silent' ) \n"
" \n"
"       site_name(text)  : Site Name \n"
" \n"
"       code(numeric)    : result code \n"
"           If code is 0, the result is success; otherwise is an error. \n"
"                        -9 : cannot load shard library. \n"
" \n"
" OPTIONS : \n"
"         'silent'       : do not display messages. \n"
" \n"
" Examples : \n"
"   >> code = changeSite('lhd') \n"
"   code = \n"
"        0 \n"
"   >> changeSite lhd \n"
" \n"
" Last update : 2018-05-30 \n"
" \n"
" See also \n"
"     currentSite retrieve retrieveVersion \n"
           )
{
  const int MIN_IN_ARG = 1;
  const int MAX_IN_ARG = 2;
  const int MAX_OUT_ARG = 1;
  std::string funcn = "changeSite";
  bool opt_verbose = true;
  std::string site_name = "";
  std::string opt_key;
  
  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.site_name
    if( 0 != (code=get_arg_string( args(0), site_name )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" site_name" << site_name << "\n";
    if( 1 <  nargin ) {
      // check 2.option
      if( 0 != (code=get_arg_string( args(1), opt_key )) )  {
        code += OCT_IF_ERR_ARG2;  break;
      }
      if( opt_key == "silent" ) {
        opt_verbose = false;
      }
      else {
        code = OCT_IF_ERR_ARG2 + OCT_IF_ERR_ARG_NO_SUPPORTED_OPTION; break;
      }
    }     
    break;
  } // End of for( ;; )
    
  if( 0 == code ) {
    code  = retrieveChangeSite2(site_name.c_str(),NULL,NULL);
  }
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
  }
  return(ret_vals);
}
