#include <octave/oct.h>
#include "dbstore-oct.h"
/*
DBSTORE_API int dbsAbort(int dbs_d);
*/
DEFUN_DLD (dbsAbort, args, nargout,
" Abort process for dbStore.  \n"
"   code = dbsAbort( dbs_des, ...)  \n"
" \n"
"       dbs_des(numeric)    : opened dbStore's descriptor id \n"
" \n"
"       code(numeric)       : status, 0 is success, other is error. \n"
" \n"
"  OPTIONS : \n"
"       'verbose' : If an error occurs, output messages to display. \n"
" \n"
"  Example : \n"
"       des = dbsOpen( .... ); \n"
"       if des > 0 \n"
"          .... \n"
"          if is_error \n"
"              dbsAbort(des); \n"
"          else \n"
"              dbsClose(des); \n"
"          end \n"
"       end \n"
" \n"
"  Last update : 2018-05-30 \n"
" \n"
"  See also \n"
"      dbsOpen dbsWrite dbsAbort dbsMessage dbstoreVersion \n"
"      dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame dbstore.txt \n"
           )
{
  const int MIN_IN_ARG = 1;
  const int NUM_OUT_ARG = 1;
  const std::string funcn = "dbsAbort";
  bool opt_verbose = false;
  int dbsd = 0;
  std::string opt_key;
  
  int code = 0;
  int nargin = args.length();

  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG ) { code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;  break; }
    // check 0. nargout
    if( nargout > NUM_OUT_ARG ) { code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;  break; }
    
    // check 1.dbsd
    if( 0 != (code = get_arg_int( args(0), dbsd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dbs_des:" << dbsd << "\n";
    break;
  } // End of for( ;; )
  // Option
  int xcode = 0;
  for( int x=MIN_IN_ARG; x < nargin; x++ ) {
    if( 0 == (xcode=get_arg_string( args(x), opt_key )) ) {
      if( opt_key == "verbose" ) {
        opt_verbose = true;
      }
      else {
        if( 0 == code )
          code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_NO_SUPPORTED_OPTION;
      }
    }
    else {
      if( 0 == code )
        code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
    }
  } // End of Option
  
  if( 0 == code ) {
    code  = dbsAbort(dbsd);
  }
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug ) octave_dbs_error_message(code, funcn);
  }
  return ret_vals;
}
