#include <octave/oct.h>

#define _OCT_IF_ERR_MESSAGE_
#include "dbstore-oct.h"

extern bool retrieve_debug;
const char *octave_dbs_error_string(int ecode, int& argno)
{
  int oct_err_code = 0;
  const char* mes = "";
  argno = 0;
  if( OCT_IF_CODE_BASE <= ecode ) {
    int eno = ecode;
    if( OCT_IF_ERR_ARG_OFFSET < eno ) {
      argno = ecode / OCT_IF_ERR_ARG_OFFSET;
      eno = ecode % OCT_IF_ERR_ARG_OFFSET;
    }
    if( OCT_IF_OCTAVE_API_BASE < eno && OCT_IF_ERR_BASE > eno ) {
      oct_err_code = eno - OCT_IF_OCTAVE_API_BASE;
      eno = OCT_IF_OCTAVE_API_ERROR;
    }
    if(  retrieve_debug && 0 != oct_err_code ) {
      octave_stdout << "Octave Error Code : "<< oct_err_code <<"\n";
    }
    for(int i=0;;i++) {
      if( eno  == err_table[i].code || OCT_IF_ERR_END_NO == err_table[i].code ) {
        mes = err_table[i].mes;
        break;
      }
    }
  }
  else {
    mes = dbsErrorMessage(ecode);
  }
  return mes;
}
const char *octave_dbs_error_message(int ecode, const std::string &func_name )
{
  int argno=0;
  const char* mes = octave_dbs_error_string(ecode, argno);
  std::string mes_str = mes;
  std::string arg_str = "";
  if( 0 != argno ) {
    arg_str = "arg#" + std::to_string(argno);
  }
  if( 0 != func_name.length() ) {
    octave_stdout << "Error "<< ecode <<" ("<< func_name << " " << arg_str<< ") : " << mes_str <<"\n";
  }
  else {
    octave_stdout << "Error "<< ecode << " : " << mes_str <<"\n";
  }
  return mes;
}
/*
DBSTORE_API  const char* dbsErrorMessage( int code);
*/
DEFUN_DLD (dbsMessage, args, nargout,
" Make a message from an error code of dbStore. Or display. \n"
"   dbsMessage code \n"
"   dbsMessage( code , func_name ); \n"
"   ret_text = dbsMessage(code, func_name) \n"
" \n"
"      code(numeric)    : error code \n"
"      func_name(text)  : function name \n"
" \n"
"      ret_text(text)   : error message \n"
" \n"
" Examples : \n"
"   >> mes = dbsMessage(-2) \n"
"   mes = \n"
" \n"
"   Inappropriate number of arguments \n"
" \n"
"   >> dbsMessage(-3,'testFunc'); \n"
"   Error -3 from testFunc : Illegal Diagnostics name.\n"
"   >> \n"
" \n"
"  Last update : 2018-05-30 \n"
" \n"
" See also \n"
"     dbsOpen dbsWrite dbsClose dbsAbort dbstoreVersion \n"
"     dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame dbstore.txt \n"
           )
{
  const int MIN_IN_ARG = 1;
  const int NUM_OUT_ARG = 1;
  const std::string funcn = "dbsMessage";
  std::string opt_key;
  
  int ecode=0;
  std::string func_name="";
  int code;
  int nargin = args.length();

  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG ) { code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;  break; }
    // check 0. nargout
    if( nargout > NUM_OUT_ARG ) { code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;  break; }
    
    // check 1.ecode
    if( 0 != (code = get_arg_int( args(0), ecode )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ecode:" << ecode << "\n";
    if( nargin > 1 ) {
      if( 0 != (code = get_arg_string( args(1), func_name )) )  {
        code += OCT_IF_ERR_ARG1;  break;
      }
      if( retrieve_debug ) octave_stdout << funcn <<" func_name:" << func_name << "\n";
    }
    break;
  } // End of for( ;; )

  octave_value_list ret_vals;
  if( 0 == code ) {
    if( 1 == nargout) {
      int argno = 0;
      std::string mes = octave_dbs_error_string( ecode, argno);
      ret_vals(0) = mes;
    }
    if( 0 == nargout) octave_dbs_error_message( ecode, func_name);
  }
  return ret_vals;
}
