#include <octave/oct.h>
#include <octave/oct-map.h>
#include <octave/uint32NDArray.h>
#include "dbstore-oct.h"

/*
DBSTORE_API int dbsWrite(int dbs_d, int ch_no,
			unsigned int param_cnt,unsigned int param_size, unsigned char *param_list,
			unsigned int data_length, unsigned char* data, char* data_type )
DBSTORE_API int dbsWrite2(int dbs_d, int ch_no,
			unsigned int param_cnt, char* param_item[], char* param_val[], int param_type[],
			unsigned int data_length, unsigned char* data, char* data_type )
*/
DEFUN_DLD (dbsWrite, args, nargout,
" Write channel data and parameter in dbStore.  \n"
"   code = dbsWrite( dbs_des, channel_number, param_struct, data_array, ... \n"
"             varargin) \n"
" \n"
"       dbs_des(numeric)    : opened dbStore's descriptor id \n"
"       channel_number(numeric) \n"
"                           : channel number \n"
"       param_struct(struct): struct array of channel's parameter \n"
"           struct fields are Pname(char),Pvalue(char),Ptype(numeric). \n"
"           describe it with 'struct([])' when there are not parameter to \n"
"           register. \n"
"       data_array(numeric) : data array of channel \n"
"           describe it with 'int16([])' when there are not data to  \n"
"           register. \n"
" \n"
"       code(numeric)       : status, 0 is success, other is error. \n"
" \n"
"  OPTIONS : \n"
"       'imagetype',image_type \n"
"                           : The data type of channel data \n"
"           Please refer to 'Note :' for the details. \n"
"           When you do not give this option, it is decided from a model of \n"
"           data_array. \n"
"       'frames',frames     : number of the frames,  \n"
"                             If you want to use frame constitution. \n"
"       'verbose'           : If an error occurs, output messages to display. \n"
" \n"
"  Example : \n"
"       des = dbsOpen('xxx@nifs.ac.jp', 'DATA_X', 213, 1, 'RAW'); \n"
"       if des > 0 \n"
"          pl = initParamList(); \n"
"          pl = setParam(pl, 'offset', 10.0); \n"
"          pl = setParam(pl, 'gain', int32(2)); \n"
"          data = zeros(1000,1,'single'); \n"
"          code = dbsWrite( des, 1, pl, data,'verbose'); \n"
"          if code == 0  \n"
"             dbsClose(des); \n"
"          else \n"
"             dbsAbort(des); \n"
"          end \n"
"       end \n"
" \n"
"  Note : \n"
"      image_type(text) has the next kind. \n"
"          'INT8'  : 8bit integer. \n"
"          'INT16' : 16bit integer. \n"
"          'INT32' : 32bit integer. \n"
"          'INT64' : 64bit integer. \n"
"          'FLT32' : single precision. \n"
"          'FLT64' : double precision. \n"
"          'GRAYn' : n bits gray image.(n:8-16) ex.) GRAY8 \n"
"          'GREYn' : same 'GRAYn' \n"
"          'RGB'   : 24 bit color image. \n"
"                    There is a meaning in letter order of 'R','G','B' \n"
"          'RGBA'  : 32 bit color image. \n"
"                    There is a meaning in letter order of 'R','G','B','A' \n"
" \n"
"  Last update : 2018-05-30 \n"
" \n"
"  See also \n"
"      dbsOpen(.oct) dbsWrite(.oct) dbsClose(.oct) dbsAbort(.oct) \n"
"      initParamList(.m) setParam(.m) printParam(.m) \n"
"      dbsOpenFrame(.oct) dbsWriteFrame(.oct) dbsCloseFrame(.oct) \n" 
"      dbsWriteFrames(.oct) dbsMessage(.oct) dbstoreVersion(.oct) dbstore.txt \n"
           )
{
  const int MIN_IN_ARG = 4;
  const int NUM_OUT_ARG = 1;
  const std::string funcn = "dbsWrite";

  int dbsd;
  int ch_no;
  octave_map params_map;
  PARAMS_BASE params_p;
  octave_value data_array_val;

  std::string opt_key = "";
  bool opt_verbose = false;
  std::string image_type = ""; 
  int num_frames = 0; 
  
  int nargin = args.length();
  int code = 0;
  params_base_initial( params_p);
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG ){ code = OCT_IF_ERR_ARG_NUM_INPUT_ARG; break; }
    // check 0. nargout
    if( nargout > NUM_OUT_ARG ){ code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG; break;}
    
    // check 1.dbsd
    if( 0 != (code = get_arg_int( args(0), dbsd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" dbs_des:" << dbsd << "\n";
    // check 2.channel_number
    if( 0 != (code = get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" channel_number:" << ch_no << "\n";
      
    // check 3.param_struct
#if (OCTAVE_MAJOR_VERSION == 4) && (OCTAVE_MINOR_VERSION == 2)
    if ( !args(2).is_map() )  {
#else
    if ( !args(2).isstruct() ) {
#endif
      code = OCT_IF_ERR_ARG3 + OCT_IF_ERR_PARAMETER_TYPE_MISSMATCH_STRUCT;   break;
    }
    //    scalar_map_value s
    params_map = args(2).map_value();
    if( 0 != (code=params_from_octave( params_map, params_p)) ) {
      code += OCT_IF_ERR_ARG3; break;
    }
    // check 4.data_array(matrix)
    if(  !args(3).is_matrix_type() ) {
      code = OCT_IF_ERR_ARG4 + OCT_IF_ERR_TYPE_MISSMATCH_ARRAY;   break;
    }
    data_array_val = args(3);
    break;
  }
  // Option
  int xcode= 0;
  for( int x = MIN_IN_ARG; x < nargin; x++ ) {
    if( 0 == (xcode=get_arg_string( args(x), opt_key )) ) {
      if( opt_key == "verbose" ) {
        opt_verbose = true;
      }
      else if( opt_key == "imagetype" ) {
        x++;
        if( x < nargin ) {
          if( 0 == (xcode=get_arg_string( args(x), image_type )) ) {
            if( retrieve_debug ) octave_stdout << funcn <<" imagetype:" << image_type << "\n";
          }
          else {
            if( 0 == code )
              code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
          }
        }
        else {
          if( 0 == code )
            code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_OPTION_NON_VALUE;
        }
      }
      else if( opt_key == "frames" ) {
        x++;
        if( x < nargin ) {
          if( 0 == (xcode=get_arg_int( args(x), num_frames )) ) {
            if( retrieve_debug ) octave_stdout << funcn <<" frames:" << num_frames << "\n";
          }
          else {
            if( 0 == code )
              code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
          }
        }
        else {
          if( 0 == code )
            code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_OPTION_NON_VALUE;
        }
      }
      else {
        if( 0 == code )
          code = OCT_IF_ERR_ARG_OPT(x+1) + OCT_IF_ERR_ARG_NO_SUPPORTED_OPTION;
      }
    }
    else {
      if( 0 == code )
        code = OCT_IF_ERR_ARG_OPT(x+1) + xcode;
    }
  } // End of Option
  
    // data array
  unsigned char* byte_data = NULL;
  size_t byte_size = 0;
  for( ; 0 == code ; ) {
    if( data_array_val.is_uint8_type() ) {
      uint8NDArray data_arr = data_array_val.uint8_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
      
      size_t cnt = data_arr.numel();
      unsigned char* data = (unsigned char*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT8";
    }
    else if( data_array_val.is_uint16_type() ) {
      uint16NDArray data_arr = data_array_val.uint16_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }

      size_t cnt = data_arr.numel();
      unsigned short* data = (unsigned short*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT16";
    }
    else if( data_array_val.is_uint32_type() ) {
      uint32NDArray data_arr = data_array_val.uint32_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      unsigned int*data = (unsigned int*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT32";
    }
    else if( data_array_val.is_uint64_type() ) {
      uint64NDArray data_arr = data_array_val.uint64_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      uint64_t* data = (uint64_t *) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT64";
    }
    else if( data_array_val.is_int8_type() ) {
      int8NDArray data_arr = data_array_val.int8_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      char* data = (char*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = (signed char)data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT8";
    }
    else if( data_array_val.is_int16_type() ) {
      int16NDArray data_arr = data_array_val.int16_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      short* data = (short*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT16";
    }
    else if( data_array_val.is_int32_type() ) {
      int32NDArray data_arr = data_array_val.int32_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      int* data = (int*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT32";
    }
    else if( data_array_val.is_int64_type() ) {
      int64NDArray data_arr = data_array_val.int64_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      int64_t* data = (int64_t*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "INT64";
    }
    else if( data_array_val.is_single_type() ) {
      FloatNDArray data_arr = data_array_val.float_array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      float* data = (float*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "FLT32";
    }
    else if( data_array_val.is_double_type() ) {
      NDArray data_arr = data_array_val.array_value();
      byte_size = data_arr.byte_size();
      byte_data = new unsigned char[byte_size];
      if( NULL ==  byte_data ) { code = OCT_RTR_ERR_MEMORY_ALLOCATE; break; }
   
      size_t cnt = data_arr.numel();
      double* data = (double*) byte_data;
      for(size_t n=0;n<cnt; n++) {        data[n] = data_arr(n); }
      if( 0 == image_type.length() ) image_type = "FLT64";
    }
    else {
      code = OCT_IF_ERR_ARG_DATA_ARRAY_NOT_NUMREIC;
      break;
    }
    break;
  } // End for( ;; )
  
  if( 0 == code ) {
    if( 0 == num_frames ) {  
      code = dbsWrite2( dbsd, ch_no, params_p.paramCounts, params_p.paramItem, params_p.paramVal, params_p.paramType,
          (unsigned int)byte_size, byte_data, (char*)image_type.c_str());
    }
    else {
      int frame_x=0;
      int frame_y=0;
      for( int n=0 ; n<params_p.paramCounts; n++ ) {
        if( 0 == strcmp(params_p.paramItem[n],"AcquisitionWindowWidth") ) {
          frame_x = atoi(params_p.paramVal[n]);
          if( 0 != frame_y ) break; // search end
        }
        else if( 0 == strcmp(params_p.paramItem[n],"AcquisitionWindowHeight" ) ) {
          frame_y = atoi(params_p.paramVal[n]);
          if( 0 != frame_x ) break; // search end
        }
      }

      code = dbsOpenFrame2( dbsd, ch_no, frame_x, frame_y, (char*)image_type.c_str(), 0);
      if( 0 == code ) 
        code = dbsWriteFrames( dbsd, ch_no, 1, num_frames,
                            (unsigned int)byte_size, byte_data);
      if( 0 == code ) 
        code = dbsCloseFrame4( dbsd, ch_no, params_p.paramCounts, params_p.paramItem, params_p.paramVal, params_p.paramType);
    }
  }
  if( NULL != byte_data ) delete [] byte_data;

  params_base_free( params_p );

  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug)  octave_dbs_error_message(code, funcn);
  }
  return ret_vals;
}
