#include <octave/oct.h>
#include <octave/dNDArray.h>
#include <octave/fNDArray.h>
#include <octave/int8NDArray.h>
#include <octave/int16NDArray.h>
#include <octave/int32NDArray.h>
#include <octave/int64NDArray.h>
#include <octave/uint8NDArray.h>
#include <octave/uint16NDArray.h>
#include <octave/uint32NDArray.h>
#include <octave/uint64NDArray.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveChData, args, nargout,
"Get Channel block data with opened retrieve-Protocol.\n"
" \n"
"   [code data data_length] ...\n"
"      = mRetrieveChData( rtd, ch_no, array_bytes, image_type, ...\n"
"                          data_class, verbose)\n"
" \n"
" Command for plotting data : plot(data)\n"
           )
{
  const int MIN_IN_ARG = 4;
  const int MAX_IN_ARG = 6;
  const int MAX_OUT_ARG = 3;
  const std::string funcn = "mRetrieveChData";
  
  int rtd = 0;
  int ch_no = 0;
  int64_t array_bytes=0;
  std::string image_type;
  std::string data_class;
  bool opt_verbose = false;

  int code = 0;
  int64_t data_length =0;

  int nargin = args.length();
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.array_bytes
    if( 0 != (code=get_arg_int64( args(2), array_bytes )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" array_bytes:" << array_bytes << "\n";
    // check 4.image_type
    if( 0 != (code=get_arg_string( args(3), image_type )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" image_type:" << image_type << "\n";
    if( nargin > 4 ) {
      // check 5.data_class
      if( 0 != (code=get_arg_string( args(4), data_class )) )  {
        code += OCT_IF_ERR_ARG5;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" data_class:" << data_class << "\n";
    if( nargin > 5 ) {
      // check 6.verbose
      if( 0 != (code=get_arg_bool( args(5), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG6;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  char* data = NULL;
  octave_value_list ret_vals;
  for( ;0 == code; ) {
    int data_byte = 1;
    if( 0 == data_class.length() ) {
      data_class = cImageType2DataClass(image_type.c_str(), data_byte);
    }
    else {
      data_byte   = cClassName2bytesize( data_class.c_str() );
    }
    int64_t pdata_length=0;
    data = new char[array_bytes];
    if( NULL == data ) {
      code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
    }
    code = retrieveChData_ex(rtd , ch_no , data, array_bytes, &pdata_length);
    if( 1 < nargout && 0 == code ) {
      data_length = pdata_length/data_byte;
      if( data_class == "int8" ) {
        int8NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = data[x];
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "uint8" ) {
        uint8NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = (unsigned char)data[x];
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "int16" ) {
        int16NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((short*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "uint16" ) {
        uint16NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((unsigned short*)data)+x );
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "int32" ) {
        int32NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((int*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "uint32" ) {
        uint32NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((unsigned int*)data)+x );
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "int64" ) {
        int64NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((int64_t*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "single" ) {
        FloatNDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((float*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( data_class == "double" ) {
        NDArray dd(dim_vector(data_length,1));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((double*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
    } // End of if
    break;
  } // End of for( ;; )
  if( NULL != data ) delete [] data;

  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
    if( 1 < nargout ) ret_vals(1) = 0;
  }
  ret_vals(0) = code;
  if( 2 < nargout ) ret_vals(2) = data_length;
  return(ret_vals);
}
