#include <octave/oct.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveChInfo, args, nargout,
"Get Channel Information with opened retrieve-Protocol.\n\n"
"   [code data_bytes comp_bytes param_count data_type image_type ...\n"
"    value_len is_nframe comment] ... \n"
"        = mRetrieveChInfo( rtd, ch_no, verbose)\n"
           )
{
  const int MIN_IN_ARG = 2;
  const int MAX_IN_ARG = 3;
  const int MAX_OUT_ARG = 9;
  const std::string funcn = "mRetrieveChInfo";
  
  int rtd = 0;
  int ch_no = 0;
  bool opt_verbose = false;
  int code = 0;
  int64_t data_bytes =0 ;
  int64_t comp_bytes = 0;
  unsigned short param_count = 0;
  short data_type = 0;
  char image_type[RTR_IMAGE_TYPE_SIZE] = {'\0'};
  unsigned short value_len = 0;
  int is_nframe = 0;
  int comment_size = RTR_COMMENT_SIZE;
  char comment[RTR_COMMENT_SIZE] = {'\0'};

  int nargin = args.length();

  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    if( nargin > 2 ) {
      // check 3.verbose
      if( 0 != (code=get_arg_bool( args(2), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG3;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  }
  if( 0 == code ) {
    char management[RTR_MANAGE_SIZE];
    code = retrieveChInfo_ex(rtd, ch_no,
			&data_bytes, &comp_bytes, &param_count,	&data_type, image_type,
      &value_len, &is_nframe, management, comment, comment_size );
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = data_bytes;
  if( 2 < nargout ) ret_vals(2) = comp_bytes;
  if( 3 < nargout ) ret_vals(3) = param_count;
  if( 4 < nargout ) ret_vals(4) = data_type;
  if( 5 < nargout ) ret_vals(5) = image_type;
  if( 6 < nargout ) ret_vals(6) = value_len;
  if( 7 < nargout ) ret_vals(7) = is_nframe;
  if( 8 < nargout ) ret_vals(8) = comment;

  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
  }

  return(ret_vals);
}
