#include <octave/oct.h>
#include <octave/dNDArray.h>
#include <octave/fNDArray.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveChV, args, nargout,
"Get Channel block data with opened retrieve-Protocol, convert to volts.\n"
" \n"
"   [code data data_length] ...\n"
"      = mRetrieveChV( rtd, ch_no, org_array_bytes, image_type, ...\n"
"                       array_type, verbose )\n"
" \n"
" Command for plotting data : plot(data)\n"
           )
{
  const int MIN_IN_ARG = 4;
  const int MAX_IN_ARG = 6;
  const int MAX_OUT_ARG = 3;
  const std::string funcn = "mRetrieveChV";
  
  int rtd = 0;
  int ch_no = 0;
  int64_t org_array_bytes = 0;
  std::string image_type;
  int array_type = 1;
  bool opt_verbose = false;
 
  int64_t data_length =0;

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.org_array_bytes
    if( 0 != (code=get_arg_int64( args(2), org_array_bytes )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" org_array_bytes:" << org_array_bytes << "\n";
    // check 4.image_type
    if( 0 != (code=get_arg_string( args(3), image_type )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" image_type:" << image_type << "\n";
    if( nargin > 4 ) {
      // check 5.array_type 1 or 2
      if( 0 != (code=get_arg_int( args(4), array_type )) )  {
        code += OCT_IF_ERR_ARG5;  break;
      }
      if( CHV_FLOAT_ARRAY_TYPE != array_type && CHV_DOUBLE_ARRAY_TYPE != array_type )  {
        code = OCT_IF_ERR_ARG5 + OCT_IF_ERR_ARG_ARRAY_TYPE;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" array_type:" << array_type << "\n";
    if( nargin > 5 ) {
      // check 6.verbose
      if( 0 != (code=get_arg_bool( args(5), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG6;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  octave_value_list ret_vals;
  void* data = NULL;
  for( ;0 == code; ) {
    int data_byte = 1;
    const char*  data_class = cImageType2DataClass(image_type.c_str(), data_byte);
    int syn_num = retrieveChVsynthesizedNumber(rtd, ch_no);
    if( 1 > syn_num ) {
      // ERROR
      if( retrieve_debug ) octave_stdout << funcn <<" retrieveChVsynthesizedNumber:" << syn_num << "\n";
      code = OCT_RTR_ERR_SYNTHESIZE_NUMBER; break;
    }
    int64_t array_size = org_array_bytes/syn_num/data_byte;
    if( CHV_FLOAT_ARRAY_TYPE == array_type ) data = new float[array_size];
    else                  data = new double[array_size];
    if( NULL == data ) {
      code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
    }
    code = retrieveChV_ex(rtd , ch_no , data, array_size, &data_length, array_type);
    if( 1 < nargout ) {
      if( 0 == code ) {
        if( CHV_FLOAT_ARRAY_TYPE == array_type ) {
          FloatNDArray dd(dim_vector(data_length,1));
          for( int64_t x=0;x<data_length;x++ ) dd(x) = *((float*)data+x);
          ret_vals(1) = dd;
        }
        else {
          NDArray dd(dim_vector(data_length,1));
          for( int64_t x=0;x<data_length;x++ ) dd(x) = *((double*)data+x);
          ret_vals(1) = dd;
        }
      }
    }
    break;
  } // End of for(;;)
  
  if( NULL != data ) {
    if( CHV_FLOAT_ARRAY_TYPE == array_type ) delete [] (float*)data;
    else                  delete [] (double*)data;
  }
  
  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
    if( 1 < nargout ) ret_vals(1) = 0;
  }
  ret_vals(0) = code;
  if( 2 < nargout ) ret_vals(2) = data_length;

  return(ret_vals);
}
