#include <octave/oct.h>
#include <octave/dNDArray.h>
#include <octave/fNDArray.h>
#include <octave/int8NDArray.h>
#include <octave/int16NDArray.h>
#include <octave/int32NDArray.h>
#include <octave/int64NDArray.h>
#include <octave/uint8NDArray.h>
#include <octave/uint16NDArray.h>
#include <octave/uint32NDArray.h>
#include <octave/uint64NDArray.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveFrameData, args, nargout,
"Get frame data in channel with opened retrieve-Protocol.\n"
" \n"
"   [code data data_length] ...\n"
"      = mRetrieveFrameData( rtd, ch_no, frame_no, frame_x, frame_y, ...\n"
"                           image_type, verbose)\n"
" \n"
" Command for displaying the image : imshow(data) or image(data)\n"
           )
{
  const int MIN_IN_ARG = 6;
  const int MAX_IN_ARG = 7;
  const int MAX_OUT_ARG = 3;
  const std::string funcn = "mRetrieveFrameData";
  
  int rtd = 0;
  int ch_no = 0;
  int frame_no = 0;
  int frame_x = 0;
  int frame_y = 0;
  std::string image_type;
  bool opt_verbose = false;

  int64_t data_bytes =0 ;

  int nargin = args.length();
  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.frame_no
    if( 0 != (code=get_arg_int( args(2), frame_no )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_no:" << frame_no << "\n";
    // check 4.frame_x
    if( 0 != (code=get_arg_int( args(3), frame_x )) )  {
      code += OCT_IF_ERR_ARG4;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_x:" << frame_x << "\n";
    // check 5.frame_y
    if( 0 != (code=get_arg_int( args(4), frame_y )) )  {
      code += OCT_IF_ERR_ARG5;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_y:" << frame_y << "\n";
    // check 6.image_type
    if( 0 != (code=get_arg_string( args(5), image_type )) )  {
      code += OCT_IF_ERR_ARG6;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" image_type:" << image_type << "\n";
    if( nargin > 6 ) {
      // check 7.verbose
      if( 0 != (code=get_arg_bool( args(6), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG7;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  octave_value_list ret_vals;
  char * data = NULL;
  for( ;0 == code; ) {
    int data_byte = 0;
    const char* data_class = cImageType2DataClass(image_type.c_str(), data_byte);
    int64_t array_bytes =  frame_x * frame_y * data_byte;
    data = new char[array_bytes];
    if( NULL == data ) {
      code = OCT_RTR_ERR_MEMORY_ALLOCATE; break;
    }
    int64_t data_length = 0;
    code = retrieveFrameData_ex(rtd, ch_no, frame_no,
			data, array_bytes, &data_bytes );
    if( 1 < nargout && 0 == code ) {
      data_length = data_bytes/data_byte;
      if( 0 == strcmp(data_class , "int8") ) {
        int8NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = data[x];
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "uint8") ) {
        uint8NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = (unsigned char)data[x];
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "int16") ) {
        int16NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((short*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "uint16") ) {
        uint16NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((unsigned short*)data)+x );
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "int32") ) {
        int32NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((int*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "uint32") ) {
        uint32NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((unsigned int*)data)+x );
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "int64") ) {
        int64NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((int64_t*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "single") ) {
        FloatNDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((float*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      else if( 0 == strcmp(data_class , "double") ) {
        NDArray dd(dim_vector(frame_x,frame_y));
        for( int64_t x=0;x<data_length;x++ ) {
          dd(x) = *( ((double*)data)+x ) ;
        }
        ret_vals(1) = dd;
      }
      if( 2 < nargout) ret_vals(2) = data_length;
    } // End of if( 1 < nargout && 0 == code )
    break;
  } // End of for( ;; )
  if( NULL != data ) delete [] data;

  ret_vals(0) = code;

  if( 0 != code ) {
    if( 1 < nargout ) ret_vals(1) = 0;
    if( 2 < nargout ) ret_vals(2) = 0;
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
  }

  return(ret_vals);
}
