#include <octave/oct.h>
#include "retrieve-oct.h"

DEFUN_DLD (mRetrieveFrameInfo, args, nargout,
"Get frame information in channel with opened retrieve-Protocol.\n"
" \n"
"   [code data_bytes image_type frame_x frame_y] ...\n"
"      = mRetrieveFrameInfo( rtd, ch_no, frame_no, verbose)\n"
           )
{
  const int MIN_IN_ARG = 3;
  const int MAX_IN_ARG = 4;
  const int MAX_OUT_ARG = 9;
  const std::string funcn = "mRetrieveFrameInfo";

  int rtd = 0;
  int ch_no = 0;
  int frame_no = 0;
  bool opt_verbose = false;
  int64_t data_bytes =0 ;
  char image_type[RTR_IMAGE_TYPE_SIZE] = {'\0'};
  int frame_x = 0;
  int frame_y = 0;

  int nargin = args.length();

  int code = 0;
  for( ;; ) {
    // check 0.nargin
    if( nargin < MIN_IN_ARG || nargin > MAX_IN_ARG) {
      code = OCT_IF_ERR_ARG_NUM_INPUT_ARG;
      break;
    }
    // check 0. nargout
    if( nargout > MAX_OUT_ARG) {
      code = OCT_IF_ERR_ARG_NUM_OUTPUT_ARG;
      break;
    }
    // check 1.rtd
    if( 0 != (code=get_arg_int( args(0), rtd )) )  {
      code += OCT_IF_ERR_ARG1;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" rtd:" << rtd << "\n";
    // check 2.ch_no
    if( 0 != (code=get_arg_int( args(1), ch_no )) )  {
      code += OCT_IF_ERR_ARG2;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" ch_no:" << ch_no << "\n";
    // check 3.frame_no
    if( 0 != (code=get_arg_int( args(2), frame_no )) )  {
      code += OCT_IF_ERR_ARG3;  break;
    }
    if( retrieve_debug ) octave_stdout << funcn <<" frame_no:" << frame_no << "\n";
    if( nargin > 3 ) {
      // check 4.verbose
      if( 0 != (code=get_arg_bool( args(3), opt_verbose )) )  {
        code += OCT_IF_ERR_ARG4;  break;
      }
    }
    if( retrieve_debug ) octave_stdout << funcn <<" verbose:" << opt_verbose << "\n";
    break;
  } // End of for( ;; )
  
  if( 0 == code ) {
    code = retrieveFrameInfo_ex(rtd, ch_no, frame_no,
			&data_bytes, image_type, &frame_x, &frame_y);
  }
  // retrun process
  octave_value_list ret_vals;
  ret_vals(0) = code;
  if( 1 < nargout ) ret_vals(1) = data_bytes;
  if( 2 < nargout ) ret_vals(2) = image_type;
  if( 3 < nargout ) ret_vals(3) = frame_x;
  if( 4 < nargout ) ret_vals(4) = frame_y;

  if( 0 != code ) {
    if( opt_verbose || retrieve_debug) octave_retrieve_error_message( code, funcn);
    octave_retrieve_close(rtd);    // with IndexNoClosing();
  }

  return(ret_vals);
}
